/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.adl;

import java.net.InetAddress;

import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.rmi.registry.Registry;
import org.objectweb.fractal.util.Fractal;

/**
 * The bootstrap to be launched on each site for distributed deployment.
 * Bootstrap are registered into a Fractal registry.
 */
public final class Bootstrap
{
  /**
   * Private Constructor
   *
   */
  private Bootstrap()
  {
  }

  /**
   * Entry point. Creates a bootstrap and register it to the specified registry.
   * Two arguments can be passed as parameters:
   * <ul>
   * <li>the name of the registry: <code>-registry registry_name</code></li>
   * <li>the name under which the bootstrap must be registered:
   * <code>-name bootstrap_name</code></li>
   * </ul>
   * Note that if <code>registry_name</code> is not specified, a local
   * registry is created.
   * 
   * @param args the name of name of the registry and the name under which this
   *          bootstrap must be registered.
   * @throws Exception if an error occurs.
   */
  public static void main(final String[] args) throws Exception
  {
    String registryName = null;
    String bootstrapName = null;

    NamingService registry;

    for (int i = 0; i < args.length; i++)
    {
      if (args[i].equals("-registry"))
      {
        registryName = args[++i];
      }
      else if (args[i].equals("-name"))
      {
        bootstrapName = args[++i];
      }
      else
      {
        System.err.println("Unknown argument : " + args[i]);
      }
    }

    if (bootstrapName == null)
    {
      bootstrapName = InetAddress.getLocalHost().getHostName();
    }

    // the RMI class loader does not work if there is no security manager
    System.setSecurityManager(new SecurityManager());

    if (registryName == null)
    {
      System.out.println("Creates a local Registry");
      Registry.createRegistry(Registry.DEFAULT_PORT);
      registry = Registry.getRegistry();
    }
    else
    {
      System.out.println("Retreive remote registry : " + registryName);
      registry = Registry.getRegistry(registryName);
    }
    registry.bind(bootstrapName, Fractal.getBootstrapComponent());
    System.out.println("Bootstrap registered under " + bootstrapName);
  }

}