/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): Pierre garcia
 */

package org.objectweb.dream.adl.nodes;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.AbstractLoader;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.adl.Node;
import org.objectweb.fractal.adl.components.Component;
import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.nodes.VirtualNode;
import org.objectweb.fractal.adl.nodes.VirtualNodeContainer;
import org.objectweb.fractal.rmi.registry.NamingService;

/**
 * This class defines a loader that lookups virtual nodes referenced in the AST.
 */
public class RemoteBootstrapLoader extends AbstractLoader
{

  /**
   * @see org.objectweb.fractal.adl.Loader#load(java.lang.String, java.util.Map)
   */
  @SuppressWarnings("unchecked")
  public Definition load(final String name, final Map context)
      throws ADLException
  {
    NamingService ns = (NamingService) context.get("naming-service");
    Definition d = clientLoader.load(name, context);
    checkNode((Node) d, ns, context);
    return d;
  }

  /**
   * Check Node
   * @param node
   * @param ns
   * @param context
   * @throws ADLException
   */
  private void checkNode(final Node node, final NamingService ns,
      final Map<String, Object> context) throws ADLException
  {
    if (node instanceof VirtualNodeContainer)
    {
      VirtualNode vn = ((VirtualNodeContainer) node).getVirtualNode();
      if (vn != null)
      {
        String name = vn.getName();
        if (context.get(name) == null)
        {
          if (ns == null)
          {
            throw new ADLException(
                "Unable to find the naming service in the context");
          }
          System.out.println("looking for " + name);
          Object bootstrap = ns.lookup(name);
          if (bootstrap == null)
          {
            throw new ADLException(
                "Unable to find the bootstrap component for virtual node "
                    + name);
          }
          context.put(name, bootstrap);
        }
      }
    }
    if (node instanceof ComponentContainer)
    {
      ComponentContainer container = (ComponentContainer) node;
      Component[] tab = container.getComponents();
      for (int i = 0; i < tab.length; i++)
      {
        Component subComponent = tab[i];
        checkNode((Node) subComponent, ns, context);
      }
    }
  }

}