/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Pierre Garcia
 */

package org.objectweb.dream.adl.reconf;

import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Compiler;
import org.objectweb.fractal.adl.Definition;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.task.core.TaskMap;

/**
 * A simple compiler with two {@link org.objectweb.fractal.adl.Compiler }client
 * interfaces :<code>precompile</code> and <code>compile</code>. These two
 * interfaces are called sequencially.
 */
@FractalComponent()
@Provides(interfaces=@Interface(name = "compiler", signature = org.objectweb.fractal.adl.Compiler.class))
public class PrecompilerCompiler implements Compiler
{

  /** The name of the client interface used to precompile */
  public static final String PRECOMPILE_ITF_NAME = "pre-compiler";
  /** The name of the client interface used to compile */
  public static final String COMPILE_ITF_NAME    = "post-compiler";
  
  @Requires(name = PrecompilerCompiler.PRECOMPILE_ITF_NAME)
  protected Compiler         precompileItf;
  @Requires(name = PrecompilerCompiler.COMPILE_ITF_NAME)
  protected Compiler         compileItf;

  /**
   * @see org.objectweb.fractal.adl.Compiler#compile(org.objectweb.fractal.adl.Definition,
   *      org.objectweb.fractal.adl.TaskMap, java.util.Map)
   */
  public void compile(Definition definition, TaskMap tasks, Map<Object,Object> context)
      throws ADLException
  {
    precompileItf.compile(definition, tasks, context);
    compileItf.compile(definition, tasks, context);
  }

  
}