/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.adl.reconf;

import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Factory;
import org.objectweb.fractal.adl.FactoryFactory;

/**
 * Reconfiguration tool factory.
 */
public final class ReconfigurationFactory
{

  /** Default Factory ADL name */
  public static final String RECONF_FACTORY_NAME    = "org.objectweb.dream.adl.reconf.ReconfFactory";

  /** Fractal Factory backend */
  public static final String RECONF_FRACTAL_BACKEND = "org.objectweb.dream.adl.reconf.ReconfFractalBackend";

  static Reconfiguration     reconf          = null;

  /**
   * Private constructor
   *
   */
  private ReconfigurationFactory()
  {
  }

  /**
   * Returns the {@link Reconfiguration }reconfiguration interface of the
   * factory.
   * 
   * @return the {@link Reconfiguration }reconfiguration interface of the
   *         factory.
   * @throws ADLException if an error occurs.
   */
  public static Reconfiguration getReconfiguration() throws ADLException
  {
    if (reconf == null)
    {
      Map<String, String> context = new HashMap<String, String>();
      context.put(FactoryFactory.BACKEND_PROPERTY_NAME, RECONF_FRACTAL_BACKEND);
      Factory f = org.objectweb.fractal.adl.FactoryFactory.getFactory();
      Map c = (Map) f.newComponent(RECONF_FACTORY_NAME, context);
      reconf = (Reconfiguration) c.get("reconfiguration");
    }
    return reconf;
  }
}