/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Pierre Garcia
 */

package org.objectweb.dream.adl.reconf.legacy;

import java.util.List;
import java.util.Map;

import org.objectweb.fractal.adl.ADLException;
import org.objectweb.fractal.adl.Node;

import org.objectweb.fractal.adl.components.ComponentContainer;
import org.objectweb.fractal.adl.components.PrimitiveCompiler;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.task.core.Task;
import org.objectweb.fractal.task.core.TaskMap;
import org.objectweb.fractal.task.deployment.lib.AbstractInstanceProviderTask;

/**
 * Compiler that register a
 * {@link org.objectweb.deployment.scheduling.component.api.InstanceProviderTask }
 * for legacy component instances.
 */
@FractalComponent()
@Provides(interfaces = @Interface(name = "compiler", signature = org.objectweb.fractal.adl.components.PrimitiveCompiler.class))
public class LegacyImplementationCompiler implements PrimitiveCompiler
{

  /**
   * Name of the mandatory interface bound to the {@link LegacyComponentBuilder}
   * used by this compiler.
   */
  public static final String       BUILDER_BINDING = "builder";

  @Requires(name = LegacyImplementationCompiler.BUILDER_BINDING)
  protected LegacyComponentBuilder builder;

  /**
   * @see PrimitiveCompiler#compile(List, ComponentContainer, TaskMap, Map)
   */
  public void compile(List path, ComponentContainer container, TaskMap tasks,
      Map context) throws ADLException
  {
    Object legacyInstance = ((Node) container).astGetDecoration("legacy");
    if (legacyInstance != null)
    {
      tasks.addTask("create", container, new LegacyInstanceProviderTask(
          legacyInstance, builder));
    }
  }

  static class LegacyInstanceProviderTask extends AbstractInstanceProviderTask
  {

    private LegacyComponentBuilder builder;

    /**
     * Creates a instance provider task for the given legacy component.
     * 
     * @param legacyComponent the legacy component, this task is instance
     *            provider.
     */
    public LegacyInstanceProviderTask(Object legacyComponent,
        LegacyComponentBuilder builder)
    {
      setInstance(legacyComponent);
      this.builder = builder;
    }

    /**
     * @see Task#execute(Map)
     */
    public void execute(Map<Object, Object> ctx) throws Exception
    {
      // stop the legacy component.
      // builder.stopComponent(getInstance(), ctx);
      builder.stopComponent(getInstance(), ctx);
    }
  }
}